@@ -12,15 +12,21 @@ import sys
 import time
 from functools import wraps
 from socketserver import ThreadingMixIn
+from urllib import parse
 from wsgiref.simple_server import make_server, WSGIServer, WSGIRequestHandler
+from xml.etree import ElementTree
 
 from Cryptodome.Protocol.KDF import scrypt
 from Cryptodome.Random import get_random_bytes
 from bottle import Bottle, abort, redirect, request, static_file, HTTPError
+from bs4 import BeautifulSoup
 
 import feedcrawler.external_tools.myjd_api
+from feedcrawler.external_sites.feed_search.shared import add_decrypt_instead_of_download
 from feedcrawler.external_sites.web_search.shared import search_web
 from feedcrawler.external_tools.myjd_api import TokenExpiredException, RequestTimeoutException, MYJDException
+from feedcrawler.external_tools.plex_api import get_client_id
+from feedcrawler.external_tools.plex_api import get_plex_headers
 from feedcrawler.providers import version, shared_state
 from feedcrawler.providers.common_functions import Unbuffered
 from feedcrawler.providers.common_functions import decode_base64
@@ -52,6 +58,8 @@ from feedcrawler.providers.myjd_connection import set_enabled
 from feedcrawler.providers.notifications import notify
 from feedcrawler.providers.sqlite_database import FeedDb
 from feedcrawler.providers.sqlite_database import ListDb
+from feedcrawler.providers.url_functions import get_url_headers, get_url
+from feedcrawler.providers.url_functions import post_url_headers
 
 
 class ThreadingWSGIServer(ThreadingMixIn, WSGIServer):
@@ -170,17 +178,40 @@ def app_container():
             redirect_url = prefix + redirect_url
         return redirect(redirect_url)
 
+    @app.hook('before_request')
+    def redirect_without_trailing_slash():
+        no_trailing_slash = [
+            "/assets/",
+            "/favicon.ico",
+            ".user.js",
+            "/api/log_entry/",
+            "/api/search/",
+            "/api/download_movie/",
+            "/api/download_show/",
+            "/api/download_bl/",
+            "/api/download_s/",
+            "/api/myjd_enable/",
+            "/api/myjd_disable/",
+            "/api/myjd_move/",
+            "/api/myjd_remove/",
+            "/api/myjd_reset/",
+            "/api/myjd_retry/",
+            "/api/myjd_pause/",
+            "/api/internal_cnl/",
+            "/sponsors_helper/api/to_decrypt/",
+            "/sponsors_helper/api/f_blocked/",
+            "/sponsors_helper/replace_decrypt/",
+            "/sponsors_helper/to_download/"
+        ]
+        if not request.path.endswith('/') and not any(s in request.path for s in no_trailing_slash):
+            raise redirect(request.url + '/')
+
     if prefix:
         @app.get('/')
         @auth_basic(is_authenticated_user)
         def index_prefix():
             return redirect(prefix)
 
-    @app.get(prefix)
-    @auth_basic(is_authenticated_user)
-    def redirect_to_slash():
-        return redirect(prefix + '/')
-
     @app.get(prefix + '/sponsors_helper')
     @auth_basic(is_authenticated_user)
     def redirect_helper_to_slash():
@@ -276,6 +307,7 @@ def app_container():
             general_conf = CrawlerConfig('FeedCrawler')
             hosters = CrawlerConfig('Hosters')
             alerts = CrawlerConfig('Notifications')
+            plex = CrawlerConfig('Plex')
             ombi = CrawlerConfig('Ombi')
             overseerr = CrawlerConfig('Overseerr')
             crawljobs = CrawlerConfig('Crawljobs')
@@ -327,6 +359,10 @@ def app_container():
                         "homeassistant": alerts.get("homeassistant"),
                         "telegram": alerts.get("telegram"),
                     },
+                    "plex": {
+                        "url": plex.get("url"),
+                        "api": bool(plex.get("api")),
+                    },
                     "ombi": {
                         "url": ombi.get("url"),
                         "api": ombi.get("api"),
@@ -472,6 +508,9 @@ def app_container():
             section.save("katfile", to_str(data['hosters']['katfile']))
             section.save("ironfiles", to_str(data['hosters']['ironfiles']))
 
+            section = CrawlerConfig("Plex")
+            section.save("url", to_str(data['plex']['url']))
+
             section = CrawlerConfig("Ombi")
             ombi_url = to_str(data['ombi']['url'])
             ombi_url = ombi_url[:-1] if ombi_url.endswith('/') else ombi_url
@@ -558,6 +597,99 @@ def app_container():
         except:
             return abort(400, "Failed")
 
+    @app.get(prefix + "/api/plex_auth/")
+    @auth_basic(is_authenticated_user)
+    def get_plex_token():
+        auth_url = ""
+        try:
+            plex_config = CrawlerConfig('Plex')
+
+            if not plex_config.get('url'):
+                return (
+                    "Bitte zuerst eine valide Plex URL eintragen und <strong>speichern</strong>! <a href='../..'>Zurück</a>")
+
+            plex_config.save('pin_id', '')
+            plex_config.save('pin_code', '')
+
+            token = plex_config.get('api')
+            if token:
+                headers = get_plex_headers(token)
+                token_valid = get_url_headers('https://plex.tv/api/v2/user', headers=headers)
+                if token_valid["status_code"] == 200:
+                    return "Plex-Token ist bereits konfiguriert und gültig!"
+                if token_valid["status_code"] == 401:
+                    token = ''
+                    plex_config.save('api', token)
+                else:
+                    return abort(400, "Plex-API reagiert nicht. Token konnte nicht geprüft werden.")
+
+            origin = parse.urlparse(request.url)
+            origin_url = origin.scheme + "://" + origin.netloc
+            generate_pin = post_url_headers('https://plex.tv/api/v2/pins?strong=true', headers={
+                'accept': 'application/json',
+                'X-Plex-Product': 'FeedCrawler',
+                'X-Plex-Client-Identifier': get_client_id(),
+                'Origin': origin_url
+            })
+
+            if generate_pin["status_code"] == 201:
+                response = json.loads(generate_pin["text"])
+                pin_id = response['id']
+                pin_code = response['code']
+                plex_config.save('pin_id', pin_id)
+                plex_config.save('pin_code', pin_code)
+            else:
+                return abort(400, "Plex-API reagiert nicht. PIN konnte nicht erzeugt werden.")
+
+            return_url = request.url.replace("/api/plex_auth/", "/api/plex_pin/")
+            client_id = get_client_id()
+            auth_url = "https://app.plex.tv/auth#" \
+                       "?clientID=" + client_id + \
+                       "&code=" + pin_code + \
+                       "&context%5Bdevice%5D%5Bproduct%5D=FeedCrawler" + \
+                       "&forwardUrl=" + parse.quote(return_url, safe='')
+
+        except:
+            time.sleep(3)
+            pass
+
+        if auth_url:
+            redirect(auth_url)
+        else:
+            return abort(400, "Failed")
+
+    @app.get(prefix + "/api/plex_pin/")
+    @auth_basic(is_authenticated_user)
+    def get_plex_pin():
+        token = ""
+        plex_config = CrawlerConfig('Plex')
+
+        try:
+            pin_id = plex_config.get('pin_id')
+            pin_code = plex_config.get('pin_code')
+
+            check_pin = get_url_headers('https://plex.tv/api/v2/pins/' + pin_id, headers={
+                'code': pin_code,
+                'X-Plex-Client-Identifier': get_client_id()
+            })
+
+            if check_pin["status_code"] == 200:
+                response = ElementTree.fromstring(check_pin["text"])
+                token = response.attrib['authToken']
+                if token and str(token) != 'null':
+                    plex_config.save('api', token)
+        except:
+            time.sleep(3)
+            pass
+
+        plex_config.save('pin_id', '')
+        plex_config.save('pin_code', '')
+
+        if token:
+            return ("Plex-Token wurde erfolgreich gespeichert! <a href='../..'>Zurück</a>")
+        else:
+            return abort(400, "Die Authentifizierung war nicht erfolgreich. Bitte erneut versuchen.")
+
     @app.get(prefix + "/api/version/")
     @auth_basic(is_authenticated_user)
     def get_version():
@@ -620,6 +752,7 @@ def app_container():
             ff = hostnames.get('ff')
             by = hostnames.get('by')
             nk = hostnames.get('nk')
+            nx = hostnames.get('nx')
             ww = hostnames.get('ww')
 
             sf = hostnames.get('sf')
@@ -635,6 +768,10 @@ def app_container():
             ff = ff.replace("f", "F", 2)
             by = by.replace("b", "B", 1)
             nk = nk.replace("n", "N", 1).replace("k", "K", 1)
+            try:
+                nx = ".".join([nx.split(".")[0].upper(), nx.split(".")[1]])
+            except:
+                pass
             ww = ww.replace("w", "W", 2)
 
             sf = sf.replace("s", "S", 1).replace("f", "F", 1)
@@ -644,7 +781,7 @@ def app_container():
 
             dd = dd.replace("d", "D", 2).replace("l", "L", 1)
 
-            bl = ' / '.join(list(filter(None, [fx, dw, hw, ff, by, nk, ww])))
+            bl = ' / '.join(list(filter(None, [fx, dw, hw, ff, by, nk, nx, ww])))
             s = ' / '.join(list(filter(None, [sf, sj])))
             f = ' / '.join(list(filter(None, [sf, ff])))
             sjbl = ' / '.join(list(filter(None, [s, bl])))
@@ -692,6 +829,8 @@ def app_container():
                 by = "Nicht gesetzt!"
             if not nk:
                 nk = "Nicht gesetzt!"
+            if not nx:
+                nx = "Nicht gesetzt!"
             if not ww:
                 ww = "Nicht gesetzt!"
             if not sj:
@@ -720,6 +859,7 @@ def app_container():
                     "ff": ff,
                     "by": by,
                     "nk": nk,
+                    "nx": nx,
                     "ww": ww,
                     "sj": sj,
                     "dj": dj,
@@ -752,6 +892,7 @@ def app_container():
                         "HW": check("HW_normal", db_status),
                         "FF": check("FF_normal", db_status),
                         "NK": check("NK_normal", db_status),
+                        "NX": check("NX_normal", db_status),
                         "WW": check("WW_normal", db_status)
                     },
                     "flaresolverr": {
@@ -763,6 +904,7 @@ def app_container():
                         "HW": check("HW_flaresolverr", db_status),
                         "FF": check("FF_flaresolverr", db_status),
                         "NK": check("NK_flaresolverr", db_status),
+                        "NX": check("NX_flaresolverr", db_status),
                         "WW": check("WW_flaresolverr", db_status)
                     },
                     "flaresolverr_proxy": {
@@ -774,6 +916,7 @@ def app_container():
                         "HW": check("HW_flaresolverr_proxy", db_status),
                         "FF": check("FF_flaresolverr_proxy", db_status),
                         "NK": check("NK_flaresolverr_proxy", db_status),
+                        "NX": check("NX_flaresolverr_proxy", db_status),
                         "WW": check("WW_flaresolverr_proxy", db_status)
                     }
                 }
@@ -1101,7 +1244,7 @@ def app_container():
             pass
         return abort(400, "Failed")
 
-    @app.post(prefix + "/api/myjd_pause/<bl>")
+    @app.post(prefix + "/api/myjd_pause/<bl>/")
     @auth_basic(is_authenticated_user)
     def myjd_pause(bl):
         try:
@@ -1573,6 +1716,64 @@ if (cnlAllowed && document.getElementsByClassName("cnlform").length) {
         except:
             return abort(400, "Failed")
 
+    @app.get(prefix + "/sponsors_helper/feedcrawler_sponsors_helper_nx.user.js")
+    @auth_basic(is_authenticated_user)
+    def feedcrawler_sponsors_helper_nx():
+        if not helper_active:
+            return abort(403, "Forbidden")
+        hostnames = CrawlerConfig('Hostnames')
+        nx = hostnames.get('nx')
+        try:
+            return """// ==UserScript==
+// @name            FeedCrawler Sponsors Helper (NX)
+// @author          rix1337
+// @description     Forwards decrypted links to FeedCrawler
+// @version         0.1.1
+// @require         https://ajax.googleapis.com/ajax/libs/jquery/3.5.1/jquery.min.js
+// @match           https://""" + nx + """/release/*
+// @grant           window.close
+// ==/UserScript==
+// Hier muss die von außen erreichbare Adresse des FeedCrawlers stehen (nicht bspw. die Docker-interne):
+const sponsorsURL = '""" + shared_state.local_address + """';
+
+function Sleep(milliseconds) {
+    return new Promise(resolve => setTimeout(resolve, milliseconds));
+}
+
+var tag = window.location.hash.replace("#", "").split('|');
+var title = tag[0];
+var password = '""" + nx + """';
+
+if (title) {
+    await Sleep(3000);
+    $('h2').prepend('<h3>[FeedCrawler Sponsors Helper] ' + title + '</h3>');
+    var dl = false;
+    var dlExists = setInterval(function() {
+        console.log($("tr:contains("+ title +")").find("a[href*=filer]").length)
+        if ($("tr:contains("+ title +")").find("a[href*=filer]").length) {
+            var link = $("tr:contains("+ title +")").find("a[href*=filer]")[0]["href"].replace("https://referer.to/?", "");
+            var links = [link];
+            console.log("[FeedCrawler Sponsors Helper] found download links: " + links);
+            clearInterval(dlExists);
+            window.open(sponsorsURL + '/sponsors_helper/to_download/' + btoa(links + '|' + title + '|' + password));
+            window.close();
+        }
+    }, 1000);
+    var tolinkExists = setInterval(function() {
+        console.log($("tr:contains("+ title +")").find("a[href*=tolink]").length)
+        if ($("tr:contains("+ title +")").find("a[href*=tolink]").length) {
+            var link = $("tr:contains("+ title +")").find("a[href*=tolink]")[0]["href"].replace("https://referer.to/?", "");
+            var links = [link];
+            console.log("[FeedCrawler Sponsors Helper] found encrypted download links: " + links);
+            clearInterval(tolinkExists);
+            window.open(sponsorsURL + '/sponsors_helper/replace_decrypt/' + btoa(links + '|' + title + '|' + password));
+            window.close();
+        }
+    }, 1000);
+}"""
+        except:
+            return abort(400, "Failed")
+
     @app.get(prefix + "/sponsors_helper/api/to_decrypt/")
     def to_decrypt_api():
         global helper_active
@@ -1609,6 +1810,7 @@ if (cnlAllowed && document.getElementsByClassName("cnlform").length) {
     def to_download(name):
         try:
             if name:
+                name.encode("ascii", errors="ignore").decode().replace("/", "").replace(" ", ".")
                 if remove_decrypt(name):
                     try:
                         notify([{
@@ -1655,6 +1857,59 @@ if (cnlAllowed && document.getElementsByClassName("cnlform").length) {
         except:
             return abort(400, "Failed")
 
+    def get_filer_folder_links(url):
+        try:
+            content = get_url(url)
+            links = []
+            if content:
+                soup = BeautifulSoup(content, 'html.parser')
+                folder_links = soup.find_all('a', href=re.compile("/get/"))
+                for link in folder_links:
+                    link = "https://filer.net" + link.get('href')
+                    if link not in links:
+                        links.append(link)
+            if links:
+                return links
+        except:
+            pass
+        return url
+
+    def clean_links(links):
+        try:
+            if len(links) == 1 and "filer.net/folder/" in links[0]:
+                links = get_filer_folder_links(links[0])
+            elif type(links) is str and "filer.net/folder/" in links:
+                if links.startswith("['") and links.endswith("']"):
+                    links = links.split("'")[1]
+                elif links.startswith('["') and links.endswith('"]'):
+                    links = links.split('"')[1]
+                links = get_filer_folder_links(links)
+        except:
+            pass
+        return links
+
+    @app.get(prefix + "/sponsors_helper/replace_decrypt/<payload>")
+    def replace_decrypt(payload):
+        try:
+            payload = decode_base64(payload.replace("%3D", "=")).split("|")
+        except:
+            return abort(400, "Failed")
+        if payload:
+            links = clean_links(payload[0])
+
+            package_name = payload[1].replace("%20", "") \
+                .encode("ascii", errors="ignore").decode().replace("/", "").replace(" ", ".")
+
+            try:
+                password = payload[2]
+            except:
+                password = ""
+
+            result = add_decrypt_instead_of_download(package_name, 'unused', [links], password, replace=True)
+            if result:
+                return "Link replaced for " + package_name
+        return abort(400, "Failed")
+
     @app.get(prefix + "/sponsors_helper/to_download/<payload>")
     def to_download(payload):
         try:
@@ -1662,8 +1917,10 @@ if (cnlAllowed && document.getElementsByClassName("cnlform").length) {
         except:
             return abort(400, "Failed")
         if payload:
-            links = payload[0]
-            package_name = payload[1].replace("%20", "")
+            links = clean_links(payload[0])
+
+            package_name = payload[1].replace("%20", "") \
+                .encode("ascii", errors="ignore").decode().replace("/", "").replace(" ", ".")
 
             try:
                 password = payload[2]
@@ -1684,8 +1941,9 @@ if (cnlAllowed && document.getElementsByClassName("cnlform").length) {
             data = request.body.read().decode("utf-8")
             payload = json.loads(data)
 
-            package_name = payload["package_name"]
-            links = payload["links"]
+            package_name = payload["package_name"] \
+                .encode("ascii", errors="ignore").decode().replace("/", "").replace(" ", ".")
+            links = clean_links(payload["links"])
 
             try:
                 password = payload["password"]
